from json import loads

class UnknownError(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class IncorrectPassword(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


class IncorrectNickname(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)

class Unauthorized(Exception):
	def __init__(*args, **kwargs):
		Exception.__init__(*args, **kwargs)


errors = {
	'Введено невірний логін або пароль.': IncorrectPassword,
	'Користувач не знайдений.': IncorrectNickname,
	0:Unauthorized,
}

def CheckException(data):
	try:
		data = loads(data)
		try:code = data['code']
		except KeyError:
			code = data['error_message']
	except:
		raise UnknownError(data)
	if code in errors: raise errors[code](data)
	else:raise UnknownError(data)


#thanks for improving objects.py -> https://github.com/GoldMasterPro

__all__ = ['Student', 'Schedule', 'Hometask', 'Timetable', 'StudentPerformance', 'LessonPerformance']


class Student:
	def __init__(self, data: dict = {}):
		self.json = data
		self.access_token = self.json.get('access_token', None)
		self.refresh_token = self.json.get('refresh_token', None)
		self.email_hash = self.json.get('email_hash', None)
		self.student_id = self.json.get('student_id', None)
		self.fio = self.json.get('FIO', None)
		self.avatar = self.json.get('avatar', None)
		self.permissions = self.json.get('permissions', None)
		self.error_message = self.json.get('error_message', None)


class Schedule:
	def __init__(self, data: dict = {}):
		self.json = data
		self.days = []
		self.error_message = self.json.get('error_message', None)
		for date in self.json.get('dates', []):
			self.days.append(self.Day(date))


	class Day:
		def __init__(self, data: dict = {}):
			self.json = data
			self.date = self.json.get('date', None)
			self.lessons = []
			for call in self.json.get('calls', []):
				self.lessons.append(self.Lesson(call))
			self.schoolSubjects = self.lessons # Обратная совместимость


		class Lesson:
			def __init__(self, data: dict = {}):
				self.json = data
				self.callId = self.json.get('call_id', None)
				self.number = self.json.get('call_number', None)

				subjects = self.json.get('subjects', [])
				subject = subjects[0] if subjects else {}

				self.subject_name = subject.get('subject_name', None)
				self.room = subject.get('room', None)
				self.distance_hometask_id = subject.get('distance_hometask_id', None)
				self.distance_hometask_is_closed = subject.get('distance_hometask_is_closed', None)
				self.hometask: list = subject.get('hometask', None)

				lesson = subject.get('lesson', [])
				lesson = lesson[0] if lesson else {}

				self.type = lesson.get('type', None)
				self.mark = lesson.get('mark', None)
				self.comment = lesson.get('comment', None)


class Hometask:
	def __init__(self, data: dict = {}):
		self.json = data
		self.hometask = self.json.get('hometask', None)
		self.answer = self.json.get('answer', None)
		self.answer_files = self.json.get('answer_files', None)
		self.is_closed = self.json.get('is_closed', None)
		self.error_message = self.json.get('error_message', None)


#new objects
class Timetable:
	def __init__(self, data: dict = {}):
		self.json = data
		#TODO THIS




class StudentPerformance:
	def __init__(self, data: dict = {}):
		self.json = data

		missed = self.json.get('missed', None)

		self.missedDays = missed.get('days', None)
		self.missedLessons = missed.get('lessons', None)
		self.error_message = self.json.get('error_message', None)
		self.subjects = list()
		for subject in self.json.get('subjects', []):
			self.subjects.append(self.Subjects(subject))

	class Subjects:
		def __init__(self, data: dict = {}):
			self.subjectId = data.get('subject_id', None)
			self.subjectName = data.get('subject_name', None)
			self.marks = data.get('marks', [])




class LessonPerformance:
	def __init__(self, data: dict = {}):
		self.json = data
		#TODO THIS


import aiohttp
from . import exceptions, objects
from json import loads
from datetime import datetime

mounth_days = {
	1: 31,
	2: 28,
	3: 31,
	4: 30,
	5: 31,
	6: 30,
	7: 31,
	8: 31,
	9: 30,
	10: 31,
	11: 30,
	12: 31,
}


def get_mounth():
	start = str(datetime.today().replace(day=1)).split(' ')[0]
	mounth_now = start.split('-')[1]
	end =  str(datetime.today().replace(day=mounth_days[int(mounth_now)])).split(' ')[0]
	return {'start': start, 'end': end}


async def post(url: str, headers, data = None):
	async with aiohttp.ClientSession() as session:
		async with session.post(url, headers=headers, data=data) as response:
			return exceptions.CheckException(await response.text()) if response.status != 200 else loads(await response.text())




class Headers:
	
	def headers(self, data = None, access_token: str = None, contentType: str = 'application/json'):

		headers = {
		'Content-Type': contentType,
		'User-Agent': 'IRC RESTClient',
		'Accept-Charset': 'utf-8, *;q=0.8',
		'Host': 'api-mobile.nz.ua',
		'Connection': 'Keep-Alive',
		'Accept-Encoding': 'gzip',
		}

		if data:headers['Content-Length'] = str(len(data))
		if access_token: headers['Authorization'] = f'Bearer {access_token}'


		return headers

	def web_headers(self):

		headers = {}

		return headers